import sys
import traceback
from pathlib import Path

from watchfiles import watch

from ssb_pubmd.core import render_article
from ssb_pubmd.core.project import create_project


def run_cli(system_arguments: list[str]) -> None:
    match system_arguments:
        case [_, "create", article_title]:
            _create_project(article_title)
        case [_, "render", file_path]:
            _render(file_path)
        case [_, "preview", file_path]:
            _preview(file_path)
        case _:
            print("Usage:")
            print("  ssb-pubmd create ARTICLE_TITLE")
            print("  ssb-pubmd render FILE_NAME")
            print("  ssb-pubmd preview FILE_NAME")
            sys.exit(1)


def _create_project(article_name: str) -> None:
    """Creates a project folder with a template file.

    The folder and file name is set to a sanitized version of the given article name.
    """
    name = create_project(article_name)
    print(f"Created project folder {name}.")
    print(f"Created template file {name}/{name}.qmd")
    print("Open the template file for further instructions.")


def _preview(file_path: str) -> None:
    _render(file_path)

    print("Watching for file changes...")
    try:
        for _changes in watch(file_path):
            print("Found changes.")
            _sync_updated_file(file_path)
            print("Watching for file changes...")
    except KeyboardInterrupt:
        print("Stopping...")


def _render(file_path: str) -> None:
    if Path(file_path).suffix != ".qmd":
        print("Only Quarto Markdown (.qmd) files are supported.")
        sys.exit(1)

    _sync_updated_file(file_path)


def _sync_updated_file(file_path: str) -> None:
    print("Syncing updated document...")
    try:
        preview_url = render_article(file_path)
        print(f"Content synced successfully. Preview URL: {preview_url}")
    except Exception:
        print("Failed to sync content. Full error log:")
        traceback.print_exc()



import os
from pathlib import Path

from ssb_pubmd.adapters.storage import LocalFileStorage
from ssb_pubmd.adapters.storage import Storage
from ssb_pubmd.core.document_publisher import ARTICLE_KEY
from ssb_pubmd.models import Factbox
from ssb_pubmd.models import Highchart


class CreateComponentError(Exception): ...


__all__ = ["Factbox", "Highchart", "create"]


def create(
    key: str,
    data: Factbox | Highchart,
) -> None:
    """Creates a component and prints a Markdown snippet to place in the article text.

    Args:
        key: A unique key for the component.
        data: An SSB component object.

    Raises:
        CreateComponentError: If reserved key is used.
    """
    if key == ARTICLE_KEY:
        raise CreateComponentError(f"Key '{ARTICLE_KEY}' is reserved.")

    if storage_file := os.environ.get("SSB_PUBMD_TEMPORARY_STORAGE"):
        storage: Storage = LocalFileStorage(Path(storage_file))
        storage.update(key, data.model_dump(exclude_none=True))

    div_config = f"{{ #{key} .ssb }}"
    div_content = ""
    if isinstance(data, Factbox):
        div_content = "\nThe factbox content is written here using Markdown syntax.\n\n"

    markdown = f"::: {div_config}\n{div_content}:::"

    print(markdown)



from dataclasses import dataclass
from typing import Literal

from ssb_pubmd.models import UserData


@dataclass
class CmsRequest:
    """A request to create or modify a content.

    To modify an existing content, `content_id` should be set.
    `target_folder` should be used for new content or as fallback when `content_id` is invalid.
    """

    target_folder: str
    user_data: UserData
    html: str = ""
    status: Literal["draft", "review"] = "draft"
    content_id: str | None = None


@dataclass
class CmsResponse:
    """A parsed response from the CMS."""

    content_path: str
    content_id: str
    preview_url: str
    html_snippet: str



import os
from typing import Protocol

from ssb_pubmd.adapters.cms_client.mimir import MimirCmsClient
from ssb_pubmd.adapters.cms_client.models import CmsRequest
from ssb_pubmd.adapters.cms_client.models import CmsResponse
from ssb_pubmd.adapters.cms_client.token_client import LocalTokenClient


class CmsClient(Protocol):
    def send(self, cms_request: CmsRequest) -> CmsResponse: ...


def get_cms_client(base_url: str, admin_path: str) -> CmsClient:
    cms_client = MimirCmsClient(base_url=base_url, admin_path=admin_path)
    if (local_token := os.environ.get("SSB_PUBMD_LOCAL_TOKEN")) is not None:
        cms_client.token_client = LocalTokenClient(local_token)
    return cms_client



import re
import sys
from importlib import resources
from pathlib import Path

from jinja2 import Template

from ssb_pubmd import templates
from ssb_pubmd.environment import cms_environment


def create_project(article_title: str) -> str:
    """Creates a project folder with a template article file.

    The folder and file name is set to a sanitized version of the given article name.
    """
    name = _sanitize_name(article_title)
    project_folder = Path.cwd() / name

    if project_folder.exists():
        print(f"Error: Folder '{name}' already exists")
        sys.exit(1)

    project_folder.mkdir()

    _create_gitignore(project_folder)
    _create_template_article(article_title, project_folder, name)

    return name


def _create_gitignore(project_folder: Path) -> None:
    """Adds a .gitignore file to the project folder.

    Prevents temporary notebooks generated by quarto to be included in version control.
    """
    with open(project_folder / ".gitignore", mode="x") as file:
        file.write("*.quarto_ipynb")


def _create_template_article(
    article_title: str, project_folder: Path, filename: str
) -> None:
    template_destination = project_folder / f"{filename}.qmd"

    with resources.open_text(templates, "quarto.qmd") as file:
        raw_template = file.read()

    rendered_template = Template(raw_template).render(
        virtual_environment_name=Path(sys.prefix).parent.name,
        article_title=article_title,
        filepath=template_destination,
        cms_admin_url=cms_environment.base_url + cms_environment.admin_path,
        project_folder=template_destination.parent.name,
        filename=template_destination.name,
    )
    template_destination.write_text(rendered_template)


def _sanitize_name(name: str) -> str:
    name = name.lower()
    name = re.sub(r"[ _]+", "-", name)
    name = re.sub(r"[^a-z0-9-]", "", name)
    name = re.sub(r"-{2,}", "-", name)
    return name.strip("-")



from typing import Any
from typing import NamedTuple
from typing import TypedDict


class BlockElement(NamedTuple):
    id: str
    metadata: dict[str, Any]
    inner_html: str


class PandocElement(TypedDict):
    t: str
    c: Any


PandocAst = TypedDict(
    "PandocAst",
    {
        "pandoc-api-version": list[int],
        "meta": dict[str, Any],
        "blocks": list[PandocElement],
    },
)



import json
import os
import subprocess

from ssb_pubmd.core.document_processor.models import PandocAst


def quarto_markdown_to_pandoc_ast(
    file_path: str, temporary_storage_filepath: str
) -> PandocAst:
    """Executes the Quarto Markdown notebook in a subprocess and converts to pandoc AST.

    When code cells needs to store data to be available after the execution, it should be stored in the temporary storage file given in the environment variable.
    """
    env = os.environ.copy()
    env["SSB_PUBMD_TEMPORARY_STORAGE"] = temporary_storage_filepath
    try:
        result = subprocess.run(
            [
                "quarto",
                "render",
                file_path,
                "--to",
                "json",
                "-M",
                "include:false",
                "--output",
                "-",
            ],
            env=env,
            text=True,
            capture_output=True,
            check=True,
        )
        return json.loads(result.stdout)  # type: ignore
    except subprocess.CalledProcessError as err:
        message = err.stderr.strip()
        raise RuntimeError(f"Quarto render failed:\n{message}") from err



import json
import subprocess
import tempfile
from collections.abc import Iterator
from dataclasses import dataclass
from pathlib import Path
from typing import Any

import pandocfilters as pf  # type: ignore

from ssb_pubmd.adapters.storage import LocalFileStorage
from ssb_pubmd.core.document_processor.models import BlockElement
from ssb_pubmd.core.document_processor.models import PandocAst
from ssb_pubmd.core.document_processor.models import PandocElement
from ssb_pubmd.core.document_processor.quarto import quarto_markdown_to_pandoc_ast


@dataclass
class Document:
    """Represents a generic document, backed by the pandoc AST.

    Attributes:
        ast: The pandoc AST of the document, as a dictionary.
        element_index: A mapping from element id's to their position in the AST.
        element_metadata: A mapping from element id's to their metadata.

            * Element metadata will typically be created programmatically in a notebook document.
            * When executing the notebook, metadata is stored in a temporary filed and retrieved after execution.

    Examples:
        ```py
        ast: PandocAst = {
            "pandoc-api-version": [1, 23, 1],
            "meta": {},
            "blocks": [
                {"t": "Div", "c": [["my-highchart", ["ssb"], [["title", "My highchart"]]], []]}
            ],
        }
        document = Document(ast=ast, element_index={}, element_metadata={})
        print(document.extract_html())
        #<div id="my-highchart" class="ssb" title="My highchart"></div>
        ```

    Reference:
        - The best "reference" of the pandoc AST is to run `pandoc FILE -t json`, where FILE is a minimal example document (e.g. in Markdown or html).
        - https://github.com/jgm/pandocfilters has some examples of how to work with the AST.
        - Note that no formal specification exists.
    """

    ast: PandocAst
    element_index: dict[str, int]
    element_metadata: dict[str, dict[str, Any]]

    @classmethod
    def from_quarto_markdown(cls, file_path: str) -> "Document":
        """Creates a document from a .qmd file."""
        with tempfile.NamedTemporaryFile(
            delete=True, suffix=".json"
        ) as temporary_storage_file:
            ast = quarto_markdown_to_pandoc_ast(file_path, temporary_storage_file.name)
            element_index = cls._generate_element_index(ast, target_class="ssb")
            storage = LocalFileStorage(Path(temporary_storage_file.name))
            element_metadata = {key: storage.get(key) for key in element_index.keys()}

            return Document(
                ast=ast,
                element_index=element_index,
                element_metadata=element_metadata,
            )

    def extract_metadata(self, target_key: str) -> dict[str, Any]:
        """Extracts the document metadata stored under a given target key."""

        def meta_to_dict(meta: Any) -> Any:
            t, c = meta.get("t"), meta.get("c")
            if t == "MetaMap":
                return {k: meta_to_dict(v) for k, v in c.items()}
            elif t == "MetaList":
                return [meta_to_dict(v) for v in c]
            else:
                return pf.stringify(c)

        return meta_to_dict(self.ast["meta"][target_key])  # type: ignore

    def extract_html(self) -> str:
        """Extracts the whole document converted to html."""
        return self._document_to_html(self.ast)

    def extract_elements(self, target_class: str) -> Iterator[BlockElement]:
        """Extracts the top-level div elements of a given target class."""
        for id_, i in self.element_index.items():
            metadata = self.element_metadata[id_]
            element = self.ast["blocks"][i]
            inner_blocks: list[PandocElement] = element["c"][1]
            inner_html = self._blocks_to_html(inner_blocks) if inner_blocks else ""
            yield BlockElement(id_, metadata, inner_html)

    def replace_element(self, id_: str, new_html: str) -> None:
        """Replaces the element of a given id with a new html fragment.

        If an element index is not created, or if the given id doesn't exist, it will raise a KeyError.
        """
        i = self.element_index[id_]
        self.ast["blocks"][i] = {
            "t": "RawBlock",
            "c": ["html", new_html],
        }

    @classmethod
    def _blocks_to_html(cls, blocks: list[PandocElement]) -> str:
        document: PandocAst = {
            "pandoc-api-version": [1, 23, 1],
            "meta": {},
            "blocks": blocks,
        }
        return cls._document_to_html(document)

    @classmethod
    def _document_to_html(cls, document: PandocAst) -> str:
        result = subprocess.run(
            ["quarto", "pandoc", "-f", "json", "-t", "html"],
            input=json.dumps(document),
            text=True,
            capture_output=True,
            check=True,
        )
        html = result.stdout
        return html

    @staticmethod
    def _generate_element_index(ast: PandocAst, target_class: str) -> dict[str, int]:
        """Creates an index of all the top-level div elements of a given target class.

        Returs:
            A mapping between element id's and their index in the document.
        """
        index = {}
        for i, element in enumerate(ast["blocks"]):
            if element["t"] != "Div":
                continue

            id_: str = element["c"][0][0]
            if not id_:
                continue

            classes: list[str] = element["c"][0][1]
            if target_class not in classes:
                continue

            index[id_] = i

        return index



from typing import Any
from typing import Literal

import narwhals as nw
from pydantic import BaseModel
from pydantic import ConfigDict
from pydantic import Field
from pydantic import field_serializer
from pydantic import field_validator


class BaseContent(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)


class Author(BaseModel):
    name: str
    email: str


class Article(BaseContent):
    """The article content type.

    Attributes:
        title: The title, placed at the top of the article.
        authors: A list of authors. Example:
            ```py
            [
                {"name": "Ola Nordmann", "email": "ola@email.com"},
                {"name": "Kari Nordmann", "email": "kari@email.com"},
            ]

        ingress: The ingress, placed below the title.
    """

    title: str = Field(min_length=1)
    authors: list[Author] = Field(default_factory=list)
    ingress: str = ""


class Factbox(BaseContent):
    """The factbox content type.

    Attributes:
        title: Title, shown when collapsed and expanded.
        display_type: How much to show when collapsed. Alternatives:

            * "default": Only the title (standard)
            * "sneakPeek": Title and some of the content
            * "aiIcon": Title and some of the content + AI-icon
    """

    title: str = Field(min_length=1)
    display_type: Literal["default", "sneakPeek", "aiIcon"] = "default"


class Highchart(BaseContent):
    """The highchart content type.

    Attributes:
        title: Title, placed above the highchart.
        graph_type: Graph type. Alternatives:

            * "line"
            * "pie"
            * "column"
            * "bar"
            * "area"
            * "barNegative"

        dataframe: A pandas, polars or pyarrow dataframe.
        tbml: URL or tbml-id.
        xlabel: The x-axis title.
        ylabel: The y-axis title.
    """

    title: str = Field(min_length=1)
    graph_type: Literal["line", "pie", "column", "bar", "area", "barNegative"] = "line"
    dataframe: Any | None = None
    tbml: str | None = None
    xlabel: str = Field(default="x", min_length=1)
    ylabel: str = Field(default="y", min_length=1)

    @field_validator("dataframe", mode="before")
    @classmethod
    def validate_dataframe(cls, v: Any) -> Any:
        if isinstance(v, dict):
            return nw.from_dict(v["df"], backend=v["backend"])
        return nw.from_native(v)

    @field_serializer("dataframe")
    def serialize_dataframe(self, v: nw.DataFrame[Any]) -> dict[str, Any]:
        return {"backend": v.implementation.value, "df": v.to_dict(as_series=False)}


UserData = Article | Factbox | Highchart



from dataclasses import dataclass
from typing import Protocol

from dapla_auth_client import AuthClient


class TokenClient(Protocol):
    def get_token(self) -> str: ...


class DaplaTokenClient:
    def get_token(self) -> str:
        return AuthClient.fetch_personal_token(audiences=["ssbno"])


@dataclass
class LocalTokenClient:
    token: str

    def get_token(self) -> str:
        """Token copied from a Dapla service to the local environment."""
        return self.token



from typing import Any
from typing import Protocol

import requests

from ssb_pubmd.common.exceptions import CmsClientError


class HttpClient(Protocol):
    def post(
        self, url: str, headers: dict[str, str], payload: dict[str, Any]
    ) -> dict[str, str]: ...


class RequestsHttpClient:
    def post(
        self, url: str, headers: dict[str, str], payload: dict[str, Any]
    ) -> dict[str, str]:
        try:
            response = requests.post(
                url,
                headers=headers,
                json=payload,
            )
            body = response.json()
        except Exception as err:
            raise CmsClientError(
                f"Sync failed. Could not parse response. Payload sent to {url}:\n{payload}"
            ) from err
        if not response.ok:
            raise CmsClientError(
                f"Sync failed. Payload sent to {url}:\n{payload}\nResponse message: {body.get('msg', 'no message')}"
            )
        return body  # type: ignore



from dataclasses import dataclass
from typing import Any

import narwhals as nw
import nh3

from ssb_pubmd.adapters.cms_client.http_client import HttpClient
from ssb_pubmd.adapters.cms_client.http_client import RequestsHttpClient
from ssb_pubmd.adapters.cms_client.models import CmsRequest
from ssb_pubmd.adapters.cms_client.models import CmsResponse
from ssb_pubmd.adapters.cms_client.token_client import DaplaTokenClient
from ssb_pubmd.adapters.cms_client.token_client import TokenClient
from ssb_pubmd.common.exceptions import CmsClientError
from ssb_pubmd.models import Article
from ssb_pubmd.models import Factbox
from ssb_pubmd.models import Highchart
from ssb_pubmd.models import UserData

DEFAULT_HTTP_CLIENT = RequestsHttpClient()
DEFAULT_TOKEN_CLIENT = DaplaTokenClient()
ARTICLE_HTML_TAGS = {
    "p",
    "br",
    "strong",
    "em",
    "b",
    "i",
    "ul",
    "ol",
    "li",
    "blockquote",
    "h2",
    "h3",
    "h4",
    "h5",
    "a",
}

FACTBOX_HTML_TAGS = ARTICLE_HTML_TAGS - {"h2"}


@dataclass
class MimirCmsClient:
    base_url: str
    admin_path: str
    http_client: HttpClient = DEFAULT_HTTP_CLIENT
    token_client: TokenClient = DEFAULT_TOKEN_CLIENT

    def send(self, cms_request: CmsRequest) -> CmsResponse:
        response_body = self.http_client.post(
            url=f"{self.base_url}/webapp/pubmd",
            headers=self._create_headers(),
            payload=self._create_payload(cms_request),
        )

        content_id = response_body.get("_id")
        content_path = response_body.get("_path")

        if content_path is None or content_id is None:
            raise CmsClientError("Sync failed. Fields missing in response body.")

        return CmsResponse(
            content_path=content_path,
            content_id=content_id,
            preview_url=self.base_url
            + self.admin_path
            + "/site/preview/default/draft"
            + content_path,
            html_snippet=self._create_html_snippet(content_id, cms_request.user_data),
        )

    def _create_headers(self) -> dict[str, str]:
        return {
            "Authorization": f"Bearer {self.token_client.get_token()}",
            "Content-Type": "application/json",
        }

    def _create_payload(self, cms_request: CmsRequest) -> dict[str, Any]:
        """Reference: https://github.com/statisticsnorway/ssbno-app-pubmd/blob/main/src/main/resources/lib/types.ts."""
        user_data = cms_request.user_data

        payload: dict[str, Any] = {
            "displayName": user_data.title,
            "parentPath": cms_request.target_folder,
        }
        if cms_request.content_id is not None:
            payload["_id"] = cms_request.content_id

        if isinstance(user_data, Article):
            payload["contentType"] = "mimir:article"
            payload["parentPath"] = self._normalize_path(cms_request.target_folder)
            payload["data"] = self._create_article_payload(user_data, cms_request.html)
        elif isinstance(user_data, Factbox):
            payload["contentType"] = "mimir:factBox"
            payload["data"] = self._create_factbox_payload(user_data, cms_request.html)
        elif isinstance(user_data, Highchart):
            payload["contentType"] = "mimir:highchart"
            payload["data"] = self._create_highchart_payload(user_data)

        return payload

    def _normalize_path(self, path: str) -> str:
        if not path.startswith("/"):
            path = "/" + path
        if not path.startswith("/ssb/"):
            path = "/ssb" + path
        return path

    def _create_article_payload(self, data: Article, html: str) -> dict[str, Any]:
        """Reference: https://github.com/statisticsnorway/mimir/blob/develop/src/main/resources/site/content-types/article/article.xml."""
        payload: dict[str, Any] = {}
        payload["authorItemSet"] = [a.model_dump() for a in data.authors]
        payload["ingress"] = data.ingress
        payload["articleText"] = nh3.clean(html, tags=ARTICLE_HTML_TAGS)
        return payload

    def _create_factbox_payload(self, data: Factbox, html: str) -> dict[str, Any]:
        """Reference: https://github.com/statisticsnorway/mimir/blob/develop/src/main/resources/site/content-types/factBox/factBox.xml."""
        payload: dict[str, Any] = {}
        payload["expansionBoxType"] = data.display_type
        payload["text"] = nh3.clean(html, FACTBOX_HTML_TAGS)
        return payload

    def _create_highchart_payload(self, data: Highchart) -> dict[str, Any]:
        """Reference: https://github.com/statisticsnorway/mimir/blob/develop/src/main/resources/site/content-types/highchart/highchart.xml."""
        payload: dict[str, Any] = {}
        payload["graphType"] = data.graph_type
        payload["xAxisTitle"] = data.xlabel
        payload["yAxisTitle"] = data.ylabel

        if data.dataframe is not None:
            payload["htmlTable"] = self._dataframe_to_html_table(data.dataframe)
        elif data.tbml is not None:
            payload["dataSource"] = {
                "_selected": "tbprocessor",
                "tbprocessor": {"urlOrId": data.tbml},
            }

        return payload

    def _dataframe_to_html_table(self, dataframe: Any) -> str:
        """Converts a dataframe to a plain html table.

        Notes:
            - The 'mimir:highchart' type expects a plain html table without special formatting, with column names in first row.

        Reference: https://github.com/statisticsnorway/mimir/blob/develop/src/main/resources/site/content-types/highchart/highchart.xml
        """
        df = nw.from_native(dataframe)

        html = "<table><tbody>"

        html += "<tr>"
        for name in df.columns:
            html += f"<td>{name}</td>"
        html += "</tr>\n"

        for row in df.iter_rows():
            html += "<tr>"
            for value in row:
                html += f"<td>{value}</td>"
            html += "</tr>"

        html += "</tbody></table>"

        return html

    def _create_html_snippet(self, content_id: str, user_data: UserData) -> str:
        """A html snippet for the content that can be parsed and rendered by Enonic CMS.

        References:
            - https://developer.enonic.com/docs/xp/stable/cms/macros#macro_instructions
            - https://github.com/statisticsnorway/mimir/blob/develop/src/main/resources/site/macros/highchart/highchart.xml
            - https://github.com/statisticsnorway/mimir/blob/develop/src/main/resources/site/macros/factBox/factBox.xml
        """
        if isinstance(user_data, Factbox):
            return f"<p>[ factBox factBox=&quot;{content_id}&quot; /]</p>"
        elif isinstance(user_data, Highchart):
            return f"<p>[ highchart highchart=&quot;{content_id}&quot; /]</p>"
        else:
            return ""



from dataclasses import asdict
from dataclasses import dataclass

from pydantic import TypeAdapter

from ssb_pubmd.adapters.cms_client import CmsClient
from ssb_pubmd.adapters.cms_client.models import CmsRequest
from ssb_pubmd.adapters.cms_client.models import CmsResponse
from ssb_pubmd.adapters.storage import Storage
from ssb_pubmd.adapters.storage import StorageError
from ssb_pubmd.core.document_processor import Document
from ssb_pubmd.models import Article
from ssb_pubmd.models import UserData

ARTICLE_KEY = "article"


@dataclass
class DocumentPublisher:
    response_storage: Storage
    cms_client: CmsClient

    def sync_document(self, document: Document) -> str:
        article_metadata = document.extract_metadata(target_key="ssb")
        user_data = Article(**article_metadata)

        cms_request = CmsRequest(
            target_folder=article_metadata["path"],
            user_data=user_data,
        )
        cms_response = self._ensure_article_exists(cms_request)

        self._sync_document_elements(document, cms_response.content_path)

        cms_request.content_id = cms_response.content_id
        cms_request.html = document.extract_html()
        cms_response = self.cms_client.send(cms_request)
        self.response_storage.update(ARTICLE_KEY, asdict(cms_response))

        return cms_response.preview_url

    def _ensure_article_exists(self, cms_request: CmsRequest) -> CmsResponse:
        stored_cms_response = self._get_stored_cms_response(ARTICLE_KEY)
        if stored_cms_response is not None:
            return stored_cms_response
        cms_response = self.cms_client.send(cms_request)
        self.response_storage.update(ARTICLE_KEY, asdict(cms_response))
        return cms_response

    def _sync_document_elements(
        self, document: Document, article_publish_path: str
    ) -> None:
        elements = document.extract_elements(target_class="ssb")
        for element in elements:
            user_data: UserData = TypeAdapter(UserData).validate_python(
                element.metadata
            )
            stored_response_data = self._get_stored_cms_response(element.id)

            cms_request = CmsRequest(
                target_folder=article_publish_path,
                user_data=user_data,
                html=element.inner_html,
            )
            if stored_response_data is not None:
                cms_request.content_id = stored_response_data.content_id
            cms_response = self.cms_client.send(cms_request)

            self.response_storage.update(key=element.id, data=asdict(cms_response))
            document.replace_element(element.id, new_html=cms_response.html_snippet)

    def _get_stored_cms_response(self, key: str) -> CmsResponse | None:
        try:
            data = self.response_storage.get(key)
            return CmsResponse(**data)
        except StorageError:
            return None



import os
import subprocess
import tempfile
from pathlib import Path

from ssb_pubmd.adapters.cms_client import CmsClient
from ssb_pubmd.adapters.cms_client import get_cms_client
from ssb_pubmd.adapters.storage import LocalFileStorage
from ssb_pubmd.core.document_processor import Document
from ssb_pubmd.core.document_publisher import DocumentPublisher
from ssb_pubmd.environment import cms_environment

cms_client = get_cms_client(cms_environment.base_url, cms_environment.admin_path)


def render_article(file_path: str, cms_client: CmsClient = cms_client) -> str:
    """Processes and syncs the article using the given Cms client, and returns a URL to the rendered preview."""
    publisher = DocumentPublisher(
        response_storage=LocalFileStorage(Path(file_path).parent / ".ssbno.json"),
        cms_client=cms_client,
    )
    document = Document.from_quarto_markdown(file_path)
    return publisher.sync_document(document)



import os
from dataclasses import dataclass
from typing import Literal


@dataclass
class CmsEnvironment:
    base_url: str
    admin_path: str
    preview_path: str
    endpoint: str


def get_cms_environment() -> CmsEnvironment:
    """Configures the environment to publish to based on the Dapla environment.

    * Dapla Dev -> ssb.no Test
    * Dapla Test -> ssb.no QA
    * Dapla Prod -> ssb.no Prod
    """
    dapla_environment: Literal["DEV", "TEST", "PROD", None] = os.environ.get(
        "DAPLA_ENVIRONMENT"
    )  # type: ignore

    admin_path = "/xp/admin"
    base_url: str
    match dapla_environment:
        case "DEV":
            base_url = "https://i.test.ssb.no"
        case "TEST":
            base_url = "https://i.qa.ssb.no"
        case "PROD":
            base_url = "https://i.ssb.no"
        case _:
            base_url = "http://localhost:8080"
            admin_path = "/admin"

    return CmsEnvironment(
        base_url=base_url,
        admin_path=admin_path,
        preview_path=f"{admin_path}/site/preview/default/draft",
        endpoint="/webapp/pubmd",
    )


cms_environment = get_cms_environment()
